var stormfront = { };
var Stormfront = { };

Stormfront.VERSION = '1.0.0';

Stormfront.Class = function(options) {
    if (this.initialize)
        this.initialize.apply(this, arguments);
};

stormfront.enableModules =  function (child, modules) {
    _.each(modules, function(module){
        stormfront.enableModules(child, module(child).dependencies);
    });
};

stormfront.extend = function(properties){
    var parent = this;
    var child = function() {
        parent.apply(this, arguments);
        stormfront.enableModules(this, properties.include);
        return this;
    };
    _.extend(child, parent);
    child.prototype = _.create(parent.prototype, properties);
    child.prototype.constructor = child;
    child.__super__ = parent.prototype;
    return child;
};

Stormfront.Class.extend = stormfront.extend;

stormfront.arguments = function(args){
    return new Stormfront.Arguments(args);
};

Stormfront.Arguments = Stormfront.Class.extend({
    initialize: function(array){
        function copy(array) {
            var copy = new Array(array.length);
            for(var i = 0; i < array.length; ++i) {
                copy[i] = array[i];
            }
            return copy;
        }
        if (array)
            this.array = copy(array);
        else
            this.array = [];
    },
    skip: function(start) {
        this.array = this.array.slice(start);
        return this;
    },
    prepend: function(item){
        this.array.unshift(item);
        return this;
    },
    at: function(index, value){
        this.array[index] = value;
        return this;
    },
    get: function(){
        return this.array;
    }
});
stormfront.hash = function(hash){
    return new Stormfront.Hash(hash);
};

Stormfront.Unchanged = {
    unchanged: 'true'
};

Stormfront.Hash = Stormfront.Class.extend({
    initialize: function(hash){
        this.hash = hash;
    },
    findNestedValue: function(keys){
        function retrieve(keys, context){
            if (!context)
                return null;
            if (keys.length === 0)
                return context;
            var current = keys.shift();
            return retrieve(keys, context[current]);
        }
        return retrieve(keys, this.hash);
    },
    subtract: function(values){
        return _.omit(this.hash, _.keys(values));
    },
    diff: function(newProperties){
        function getProperties(oldProperties, newProperties){
            var candidates = _.keys(newProperties).concat(_.keys(oldProperties));
            var position = candidates.length;
            var keys = [];

            while (position--) {
                var candidate = candidates[position];
                if (keys.indexOf(candidate) === -1)
                    keys.unshift(candidate);
            }
            return keys;
        }
        function evaluateArray(previous, current) {
            if (current.length !== previous.length)
                return current;

            var difference = _.any(previous, function(value, index){
                return evaluateObject(value, current[index]);
            });
            return difference ? current : Stormfront.Unchanged;
        }

        function evaluateValue(previous, current){
            if (_.isArray(current))
                return evaluateArray(previous, current);
            else if (_.isObject(current))
                return evaluateObject(previous, current);
            else if (previous !== current)
                return current;
            return Stormfront.Unchanged;
        }

        function evaluateObject(oldProperties, newProperties) {
            var changes = { };
            function addEntry(property, value) {
                if (value)
                    changes[property] = value;
            }

            var properties = getProperties(oldProperties, newProperties);
            _.each(properties, function(property){
                var previous = oldProperties[property];
                var current = newProperties[property];
                if (previous && current)
                    addEntry(property, evaluateValue(previous, current));
                else if (previous)
                    addEntry(property, previous);
                else
                    addEntry(property, current);
            });
            return _.isEmpty(changes) ? Stormfront.Unchanged : changes;
        }
        if (!this.hash)
            return newProperties;
        if (!newProperties)
            return this.hash;

        return evaluateObject(this.hash, newProperties);
    }
});
stormfront.number = function(number){
    return new Stormfront.Number(number);
};

Stormfront.Number = Stormfront.Class.extend({
    initialize: function(number){
        this.number = number;
    },
    isWithin: function(range){
        return range[0] < this.number && range[1] > this.number;
    },
    compare: function(value){
        return this.number < value ? -1 : this.number > value ? 1 : 0;
    }
});
Stormfront.Errors = {
    FEEDBACK: 'An unexpected error was encountered'
};

Stormfront.Response = Stormfront.Class.extend({
    initialize: function(xhr){
        this.xhr = xhr;
    },
    isActionable: function(){
        return !(this.hasBeenHandled() || this.isUserUnauthorized() || this.hasBeenAborted());
    },
    hasBeenHandled: function(){
        return this.xhr.handled === true;
    },
    markAsHandled: function(){
        this.xhr.handled = true;
    },
    isUserUnauthorized: function(){
        return this.xhr.status === 401;
    },
    hasBeenAborted: function(){
        return this.xhr.readyState != 4;
    },
    getError: function(){
        try {
            var message = JSON.parse(this.xhr.responseText);
            return message.error || message.message;
        }
        catch (e){
            console.warn('Unhandled error', this.xhr, e);
            return Stormfront.Errors.FEEDBACK;
        }
    }
});
stormfront.string = function(text){
    return new Stormfront.String(text);
};

Stormfront.String = Stormfront.Class.extend({
    initialize: function(text){
        this.text = text;
    },
    decimal: function(places, placeholder){
        placeholder = placeholder || '';
        places = places || 3;
        var candidate = parseFloat(this.text);
        return isNaN(candidate) ? placeholder : candidate.toFixed(places);
    },
    scientific: function(){
        return this.text === null || $.trim(this.text) === '' ? '' : parseFloat(this.text).toExponential(3);
    },
    firstWord: function(){
        return this.text ? this.text.split(' ')[0] : '';
    },
    title: function(){
        return this.camelCase(' ', ' ');
    },
    camelCase: function(delimiter, alimiter){
        delimiter = delimiter || ' ';
        alimiter = alimiter || '';
        function capitalize(word){
            return word.charAt(0).toUpperCase() + word.substr(1).toLowerCase();
        }
        var words = this.text.split(delimiter);
        return _.map(words, capitalize).join(alimiter)
    }
});
stormfront.time = function(datetime){
    return new Stormfront.Time(datetime);
};

Stormfront.Time = Stormfront.Class.extend({
    initialize: function(datetime){
        this.time = moment(datetime || (new Date()).getTime());
    },
    now: function(){
        return this.time;
    },
    getDiff: function(unit){
        unit = unit || 'hours';
        var now = moment().format();
        return this.time.diff(now, unit);
    },
    getTimeAgo: function(){
        return this.time.fromNow();
    },
    getLongDate: function(){
        return this.time.format('MMMM D, YYYY h:mm:ss a');
    },
    getShortDate: function(){
        return this.time.format('MMMM D, YYYY');
    }
});




stormfront.type = function(type){
    return (new Stormfront.Type(type)).get();
};
Stormfront.Type = Stormfront.Class.extend({
    initialize: function (name) {
        this.name = name;
    },
    get: function () {
        if (_.isString(this.name)) {
            var name = this.name.split('.');
            return stormfront.hash(window).findNestedValue(name);
        } else {
            return this.name;
        }
    }
});

Stormfront.Patterns = {
    When: function (subject, methods) {
        subject.when = subject.when || {};
        _.each(methods, function (method) {
            var actualMethod = subject[method];
            subject[method] = function () {
                var when = subject.when[method];
                var value = actualMethod.apply(subject, arguments);
                when && when.apply(subject, arguments);
                return value;
            }
        });
        subject.when.initialize && subject.when.initialize.apply(subject);
    },
    Dispatcher: function (subject, dispatcher) {
        subject.dispatch = function (event) {
            dispatcher.dispatch(event);
        };
        subject.getDispatcher = function () {
            return dispatcher;
        };
    },
    Events: function (subject) {
        if (!subject.listenTo)
            $.extend(subject, Backbone.Events);

        $.extend(subject, {
            forward: function (other, type) {
                subject.listenTo(other, type || 'all', subject.trigger);
            }
        })
    }
};


Stormfront.Dispatches = {
    RECEIVED: 'Dispatcher:Action:Received',
    QUEUED: 'Dispatcher:Action:Queued',
    READY: 'Dispatcher:Action:Ready',
    EXECUTING: 'Store:Action:Executing',
    EXECUTED: 'Store:Action:Executed',
    MISSING: 'Store:Action:Missing',
    ERROR: 'Store:Action:Error',
    CHANGE: 'Store:Changed',
    UNCHANGED: 'Store:Unchanged',
    NO_RETURN: 'Store:NoReturnValue'
};

Stormfront.NO_CHANGE = 'Store:Nothing_Changed';
Stormfront.Reducer = Stormfront.Class.extend({
    type: 'NOT:IMPLEMENTED',
    dependencies: [],
    initialize: function (dispatcher) {
        Stormfront.Patterns.Dispatcher(this, dispatcher);
    },
    execute: function (store, event) {
        throw 'The reducer does not implement a function body'
    }
});
Stormfront.Ajax = {
    START: 'AJAX:START',
    SUCCESS: 'AJAX:SUCCESS',
    ERROR: 'AJAX:ERROR',
    ABORTED: 'AJAX:ABORTED',
    PENDING: 'AJAX:PENDING',
    Support: {
        generateKey: function (event) {
            return event.type;
        },
        createGenericRequest: function (type) {
            return function (info) {
                return _.extend(info, {type: type});
            }
        },
        createGenericReducer: function (type) {
            return Stormfront.Ajax.RequestReducer.extend({
                type: type,
                execute: function (store, event) {
                    try {
                        event.callbacks[type].call(event.callbacks.context, store, event);
                    } catch (e) {
                        console.warn('Ajax Handler encountered an error', e);
                    }
                    return this.updateEntry(store, event, type);
                }
            });
        }
    }
};

Stormfront.Ajax.RequestReducer = Stormfront.Reducer.extend({
    getKey: function (event) {
        return Stormfront.Ajax.Support.generateKey(event);
    },
    getEntries: function (store) {
        return store.ajax || (store.ajax = { });
    },
    getEntry: function (store, event) {
        var key = this.getKey(event);
        var entries = this.getEntries(store);
        return entries[key] || (entries[key] = { });
    },
    updateEntry: function (store, event, state) {
        var ajax = this.getEntry(store, event);
        ajax.state = state;
        ajax.request = event.xhr;
        ajax.exception = event.exception;
        return store;
    },
    dropEntry: function (store, event) {
        var key = this.getKey(event);
        var entries = this.getEntries(store);
        delete entries[key];
    }
});

Stormfront.Ajax.Request = Stormfront.Ajax.RequestReducer.extend({
    type: 'NOT:IMPLEMENTED',
    canExecute: function (store, event) {
        return true;
    },
    execute: function (store, event) {
        if (this.canExecute(store, event)) {
            event.callbacks = { };
            event.callbacks.context = this;
            event.callbacks[Stormfront.Ajax.START] = this.onStart;
            event.callbacks[Stormfront.Ajax.SUCCESS] = this.onSuccess;
            event.callbacks[Stormfront.Ajax.ERROR] = this.onError;

            var self = this;
            function onStart (xhr, settings) {
                _.extend(event, { xhr: xhr });
                self.dispatch(new Stormfront.Ajax.StartRequest(event));
            }
            function onSuccess (data, status, xhr) {
                _.extend(event, {
                    data: data,
                    status: status,
                    xhr: xhr
                });
                self.dispatch(new Stormfront.Ajax.SuccessRequest(event));
            }
            function onError (xhr, status, exception) {
                _.extend(event, {
                    status: status,
                    xhr: xhr,
                    exception: exception
                });
                self.dispatch(new Stormfront.Ajax.ErrorRequest(event));
            }
            var request = this.getParameters(store, event);
            var settings = _.extend(request, {
                beforeSend: onStart,
                error: onError,
                success: onSuccess
            });
            this.updateEntry(store, event, Stormfront.Ajax.PENDING);
            this.send(settings);
            return store;
        }
        return Stormfront.NO_CHANGE;
    },
    send: function (settings) {
        // TODO: Remaining Concerns AJAX:
        // TODO:    Aborting similar request
        // TODO:    Avoiding equal request
        $.ajax(settings);
    },
    getParameters: function (store, event) {
        return { url: '', method: 'get', data: {} }
    },
    onStart: function (store, event) { },
    onSuccess: function (store, event) { },
    onError: function (store, event) { }
});

Stormfront.Ajax.StartRequest = Stormfront.Ajax.Support.createGenericRequest(Stormfront.Ajax.START);
Stormfront.Ajax.Start = Stormfront.Ajax.Support.createGenericReducer(Stormfront.Ajax.START);

Stormfront.Ajax.SuccessRequest = Stormfront.Ajax.Support.createGenericRequest(Stormfront.Ajax.SUCCESS);
Stormfront.Ajax.Success = Stormfront.Ajax.Support.createGenericReducer(Stormfront.Ajax.SUCCESS);

Stormfront.Ajax.ErrorRequest = Stormfront.Ajax.Support.createGenericRequest(Stormfront.Ajax.ERROR);
Stormfront.Ajax.Error = Stormfront.Ajax.Support.createGenericReducer(Stormfront.Ajax.ERROR);

Stormfront.Ajax.Reducers = [
    Stormfront.Ajax.Start,
    Stormfront.Ajax.Success,
    Stormfront.Ajax.Error
];
Stormfront.Store = Stormfront.Class.extend({
    initialize: function (options) {
        $.extend(this, Backbone.Events);

        this._state = new Stormfront.Store.State();
        this._reducers = new Stormfront.Store.Reducers(options);
    },
    executeReducer: function (action) {
        var reducer = this._reducers.find(action);
        if (reducer)
            this.tryExecuteReducer(action, reducer);
        else
            this.trigger(Stormfront.Dispatches.MISSING, action, reducer);
    },
    tryExecuteReducer: function (action, reducer) {
        try {
            this.executeExternalCode(action, reducer);
        } catch (e) {
            this.trigger(Stormfront.Dispatches.ERROR, action, reducer);
        }
    },
    executeExternalCode: function(action, reducer){
        var current = this._state.getState();
        this.trigger(Stormfront.Dispatches.EXECUTING, action, reducer, current);

        var updated = reducer.execute(current, action);
        this.trigger(Stormfront.Dispatches.EXECUTED, action, reducer, updated);

        if (updated === Stormfront.NO_CHANGE)
            this.trigger(Stormfront.Dispatches.UNCHANGED, action, reducer);
        else if (this._state.setState(updated))
            this.trigger(Stormfront.Dispatches.CHANGE, action, reducer, updated, current);
        else
            this.trigger(Stormfront.Dispatches.NO_RETURN, action, reducer);
    }
});

Stormfront.Store.State = Stormfront.Class.extend({
    setState: function (state) {
        return state ? this.updateState(state) : false;
    },
    updateState: function (state) {
        this._state = this.copyState(state);
        return true;
    },
    getState: function () {
        return this.copyState(this._state);
    },
    copyState: function (state) {
        return $.extend(true, {}, state);
    }
});

Stormfront.Store.Reducers = Stormfront.Class.extend({
    reducers: { },
    initialize: function (options) {
        function createReducer(type) {
            var reducer = new type(options.dispatcher);
            this.reducers[reducer.type] = reducer;
            _.each(reducer.dependencies, createReducer, this);
        }

        _.each(options.reducers, createReducer, this);
        _.each(Stormfront.Ajax.Reducers, createReducer, this);
        _.each(Stormfront.Store.Operations, createReducer, this);
    },
    find: function(event){
        return this.reducers[event.type];
    }
});

SetStateAction = function(state){
    return {
        type: 'Stormfront:Store:State:Set',
        state: state
    }
};

SetState = Stormfront.Reducer.extend({
    type: 'Stormfront:Store:State:Set',
    execute: function (store, action) {
        return action.state;
    }
});

Stormfront.Store.Operations = [SetState];
Stormfront.Dispatcher = Stormfront.Class.extend({
    initialize: function () {
        $.extend(this, Backbone.Events);
        var queue = [];
        var root = this;

        this.dispatch = function (event) {
            this.trigger(Stormfront.Dispatches.RECEIVED, event);
            if (_.isEmpty(queue))
                process(event);
            else
                enqueue(event);
        };

        function process(event) {
            enqueue(event);
            processQueue();
        }

        function enqueue(event) {
            queue.push(event);
            root.trigger(Stormfront.Dispatches.QUEUED, event);
        }

        function processQueue() {
            if (_.isEmpty(queue))
                return;
            processNext();
            processQueue();
        }

        function processNext() {
            root.trigger(Stormfront.Dispatches.READY, queue[0]);
            queue.shift();
        }
    }
});
Stormfront.Container = Stormfront.Class.extend({
    initialize: function (reducers) {
        Stormfront.Patterns.Events(this);
        Stormfront.Patterns.Dispatcher(this, new Stormfront.Dispatcher());
        var ursula = this.getDispatcher();
        var store = new Stormfront.Store({
            reducers: reducers,
            dispatcher: ursula
        });

        function execute (store) {
            return function (action) {
                store.executeReducer(action);
            }
        }

        this.listenTo(ursula, Stormfront.Dispatches.READY, execute(store));
        this.forward(ursula);
        this.forward(store);
    },
    start: function (initial) {
        this.dispatch(SetStateAction(initial));
    }
});

Homefront = {};
Homefront.Virtualize = require('virtual-html');
Homefront.Virtual = require('virtual-dom');
Homefront.Diff = Homefront.Virtual.diff;
Homefront.Patch = Homefront.Virtual.patch;
Homefront.Create = Homefront.Virtual.create;

Homefront.Entity = Stormfront.Class.extend({
    defaults: null,
    initialize: function (properties) {
        this.properties = _.extend(this, this.defaults, properties);
    },
    state: function(){
        return this.getEntity().state;
    },
    getEntity: function(){
        return this.properties;
    },
    isBusy: function(){
        var state = this.state();
        return state === Stormfront.Ajax.START ||
            state === Stormfront.Ajax.PENDING;
    }
});
Homefront.Collection = Homefront.Entity.extend({
    entity: null,
    state: '',
    initialize: function (items, state) {
        state = state || this.state;
        var entity = this.entity;
        this.properties = {
            state: state,
            data: _.map(items, function (option) {
                return new entity(option);
            })
        };
        this.state = state;
    },
    list: function () {
        return this.getEntity().data;
    },
    isEmpty: function () {
        return this.list().length === 0;
    },
    find: function (parameters) {
        return _.findWhere(this.list(), parameters);
    },
    groupBy: function (field) {
        return _.groupBy(this.list(), field);
    },
    pluck: function (field) {
        return _.pluck(this.list(), field);
    },
    first: function () {
        return this.list()[0];
    },
    onlyOne: function () {
        return this.list().length === 1;
    }
});
Homefront.Request = Stormfront.Ajax.Request.extend({
    getEntity: function (store, action) {
        return {};
    },
    onStart: function (store, action) {
        this.getEntity(store, action).state = Stormfront.Ajax.START;
        return store;
    },
    onSuccess: function (store, action) {
        var entity = this.getEntity(store, action);
        entity.state = Stormfront.Ajax.SUCCESS;
        entity.data = action.data;
        return store;
    },
    onError: function (store, action) {
        var entity = this.getEntity(store);
        entity.state = Stormfront.Ajax.ERROR;
        entity.error = action.error;
        return store;
    }
});

Homefront.Templating = function (template, viewModel) {
    return Handlebars.compile(template)(viewModel);
};
Homefront.Rendering = function (html, events, context) {
    return Homefront.Eventing(Homefront.Virtualize(html), events, context);
};
Homefront.Eventing = function (view, events, context) {
    function delegate (handler) {
        return function () {
            handler.apply(context, arguments);
        }
    }

    _.each(events, function (handler, key) {
        view.properties['on' + key] = delegate(handler);
    });
    return view;
};

Homefront.Base = Stormfront.Class.extend({
    initialize: function (dispatcher) {
        Stormfront.Patterns.Dispatcher(this, dispatcher);
        Stormfront.Patterns.When(this, ['initialize', 'render', 'attached']);
    },
    render: function(properties){
        // return a v-tree to have
        // it rendered to the screen
        this.properties = properties;
        return properties;
    },
    attached: function (dom) {
        this.dom = dom;
    }
});
Homefront.View = Homefront.Base.extend({
    template: '<div></div>',
    templating: Homefront.Templating,
    rendering: Homefront.Rendering,

    render: function (properties) {
        this.properties = properties;
        this.context = this.getContext(properties);
        this.html = this.templating(this.template, this.context);
        this.setNode(this.rendering(this.html, this.events, this));
        return this.getNode();
    },
    getContext: function (properties) {
        return properties;
    },
    getNode: function(){
        return this.node;
    },
    setNode: function(node){
        this.node = node;
    },

    attached: function (dom) {
        this.dom = dom;
    }
});

Homefront.Guidance = {
    Generic: function (type, text) {
        return Homefront.View.extend({
            template: '<{{type}} class="guidance">{{text}}</{{type}}>',
            getContext: function () {
                return {type: type, text: text}
            }
        });
    },
    Paragraph: function (text) {
        return Homefront.Guidance.Generic('p', text);
    },
    Table: function (text) {
        text = '<td colspan="99">' + text + '</td>';
        return Homefront.Guidance.Generic('tr', text);
    },
    List: function (text) {
        return Homefront.Guidance.Generic('li', text);
    }
};

Homefront.Empty = { tagName: '', children: [] };

Homefront.VirtualGateway = function(view) {
    function getProperties(node){
        return node ? node.properties || {} : {};
    }
    function getClass(node){
        return getProperties(node).class || '';
    }
    function hasClass(node, className){
        return (_.contains(getClass(node).split(' '), className));
    }
    function find(node, className){
        if (hasClass(node, className))
            return node;
        var candidate = Homefront.Empty;
        _.find(node.children, function(child){
            candidate = find(child, className);
            return candidate !== Homefront.Empty;
        });
        return candidate;
    }
    function findParent(node, className){
        var candidate = Homefront.Empty;
        _.find(node.children, function(child){
            if (hasClass(child, className))
                candidate = node;
            else
                candidate = findParent(child, className);
            return candidate !== Homefront.Empty;
        });
        return candidate;
    }
    return _.extend(view, {
        hasClass: function(className){
            return hasClass(view.getNode(), className);
        },
        find: function (location) {
            return find(view.getNode(), location);
        },
        parent: function(location) {
            return findParent(view.getNode(), location);
        },
        replace: function (location, node) {
            var child = find(view.getNode(), location);
            var parent = findParent(view.getNode(), location);
            var index = _.indexOf(parent.children, child);
            parent.children[index] = node;
        },
        append: function(node, location){
            if (location)
                find(view.getNode(), location).children.push(node);
            else
                view.getNode().children.push(node);
        }
    });
};

Homefront.Chaperone = function(child){
    _.extend(child, {
        default: Homefront.Guidance.Paragraph,
        createChild: function (name) {
            var type = stormfront.type(name);
            return (type) ?
                new type(this.getDispatcher()) :
                this.guidance('View was not found: ' + name);
        },
        renderChild: function (child, properties) {
            return child.render(properties || this.properties);
        },
        appendChild: function (child, location) {
            this.append(child, location);
        },
        attachChild: function (child, location) {
            this.replace(location, child);
        },
        //todo: when a view wants to render two children

        guidance: function (text) {
            return new (this.default(text))();
        }
    });

    return {
        dependencies: [Homefront.VirtualGateway]
    }
};
Homefront.Layout = Homefront.View.extend({
    include: [Homefront.Chaperone],
    children: [],
    when: {
        render: function () {
            this.appendChildren(this.children);
        }
    },
    appendChildren: function(types){
        _.each(types, function(type){
            var child = this.createChild(type);
            var view = this.renderChild(child);
            this.appendChild(view);
        }, this);
    }
});

Homefront.Template = Homefront.View.extend({
    include: [Homefront.Chaperone],
    children: {},
    when: {
        render: function () {
            this.attachChildren(this.children);
        }
    },
    attachChildren: function(types){
        _.each(types, function(type, location){
            var child = this.createChild(type);
            var view = this.renderChild(child);
            this.attachChild(view, location);
        }, this);
    }
});
Homefront.Switch = Homefront.Base.extend({
    cases: {},
    include: [Homefront.Chaperone],
    render: function (properties) {
        this.properties = properties;
        return this.createCase(this.getState(properties)).render(properties);
    },
    getState: function (properties) {
        return properties;
    },
    createCase: function(state){
        var candidate = this.cases[state];
        return (candidate) ?
            this.createChild(candidate) :
            this.guidance('Case was not found for state: ' + state);
    }
});
Homefront.Collection = Homefront.Switch.extend({
    loading: null,
    item: null,
    empty: null,
    error: null,
    when: {
        initialize: function(){
            this.cases = {
                loading: this.loading,
                error: this.error,
                populated: Homefront.List.extend({
                    template: this.template,
                    empty: this.empty,
                    item: this.item,
                    selectList: this.selectList
                })
            }
        }
    },
    getState: function (properties) {
        switch(properties.state) {
            case Stormfront.Ajax.START:
                return 'loading';
            case Stormfront.Ajax.PENDING:
                return 'loading';
            case Stormfront.Ajax.SUCCESS:
                return 'populated';
            case Stormfront.Ajax.ERROR:
                return 'error';
            case Stormfront.Ajax.ABORTED:
                return 'error';
            default:
                return properties.state;
        }
    },
    selectList: function (properties) {
        return properties.list;
    }
});
Homefront.List = Homefront.View.extend({
    include: [Homefront.Chaperone],
    item: null,
    empty: null,
    when: {
        render: function () {
            this.renderList(this.selectList(this.properties));
        }
    },
    selectList: function (properties) {
        return properties;
    },
    renderList: function (list) {
        (_.isEmpty(list)) ?
            this.appendChild(this.renderChild(this.createChild(this.empty))) :
            _.each(list, this.renderEntry, this);
    },
    renderEntry: function (entry) {
        this.appendChild(this.renderChild(this.createChild(this.item), entry));
    }
});

Homefront.Text = Homefront.View.extend({

});

Homefront.Label = Homefront.Text.extend({

});

Homefront.Paragraph = Homefront.Text.extend({

});

Homefront.Input = Homefront.Text.extend({

});

Homefront.Select = Homefront.Input.extend({

});

Homefront.TextBox = Homefront.Input.extend({

});

Homefront.Clicker = Homefront.Input.extend({

});

Homefront.Button = Homefront.Clicker.extend({

});
Homefront.Page = Stormfront.Class.extend({
    initial: {},
    reducers: null,
    view: null,
    initialize: function(location){
        Stormfront.Patterns.Events(this);
        this.start(this.reducers, location, this.view, this.initial);
    },
    start: function(reducers, location, view, initial){
        var container = new Stormfront.Container(reducers);
        var options = {
            location: location,
            dispatcher: container.getDispatcher(),
            view: this.view
        };
        this.listenTo(container, Stormfront.Dispatches.CHANGE, this.renderView(options));
        container.start(initial);
    },
    renderView: function(options){
        return function(action, reducer, newProperties, oldProperties) {
            var view = new this.view(options.dispatcher);
            var newTree = view.render(newProperties, oldProperties, action, reducer);
            this.attachView(newTree, options.location);
        }
    },
    attachView: function(newTree, location){
        this.tree = this.tree ?
            this.updateView(newTree) :
            this.createView(location, newTree);
        view.transition('attached');
    },
    createView: function(location, newTree){
        this.root = Homefront.Create(newTree);
        location.appendChild(this.root);
        return newTree;
    },
    updateView: function(newTree){
        var patches = Homefront.Diff(this.tree, newTree);
        this.root = Homefront.Patch(this.root, patches);
        return newTree;
    }
});
